
#include <windows.h>
#include <winsock.h>
#include <errno.h>
#include <assert.h>
#include <stdio.h>
#include <string>

#define ASSERT assert

/** Initializes windows sockets.
 */
bool initializeWindowsSockets()
{
	WORD version = MAKEWORD(1, 1);
	WSADATA wsaData;
	return (WSAStartup(version, &wsaData) == 0);
}

/** Un-initializes windows sockets.
 */
void uninitializeWindowsSockets()
{
	WSACleanup();
}

/** Connects to the given machine at the given port.
 */
SOCKET connectTo(CONST char *hostName, int port) // throws std::string
{
	ASSERT(hostName && *hostName);
	ASSERT(port > 0);

	// Create the socket
	SOCKET socket;
	{
		socket = ::socket(AF_INET, SOCK_STREAM, 0);
		ASSERT(socket != INVALID_SOCKET);
		if (socket == INVALID_SOCKET)
			throw std::string("Unable to create a socket");
	}

	// Format the address to connect to
	struct sockaddr_in address;
	{
		address.sin_family = AF_INET;
		unsigned int ipAddress = inet_addr(hostName);
		if (ipAddress != INADDR_NONE)
			memcpy(&address.sin_addr, &ipAddress, 4); // already 
		else {
			struct hostent *entry = gethostbyname(hostName);
			ASSERT(entry != NULL);
			if (entry == NULL)
				throw std::string("Unable to resolve host name");
			memcpy(&address.sin_addr, entry->h_addr, entry->h_length);
		}
		address.sin_port = htons(port);
	}

	// Connect
	{
		if (connect(socket, (struct sockaddr *) &address, sizeof(address)) == SOCKET_ERROR) {
			return INVALID_SOCKET;
		}
	}

	return socket;
}

/** Macro for printing output to stdout and the Windows debug console.
 * NB: __format should contain %s and %d, in that order!
 */
#define DUMP_OUT(__format) \
	{ \
		char buf[1024] = {0}; \
		sprintf(buf, __format, hostName.c_str(), port); \
		OutputDebugString(buf); \
		fprintf(stdout, __format, hostName.c_str(), port); \
	}

/** Opens a socket to a given host and port.
 * Returns 0 if successful, 1 if there was a failure to connect to the target machine, -1 if there was an internal error.
 */
int tryToConnect(const std::string& hostName, unsigned short port)
{
	SOCKET socket;

	try {
		socket = connectTo(hostName.c_str(), port);
	}
	catch (const std::string& errorMessage) {
		fprintf(stdout, "ConnectToSocketSanityCheck: Internal error: %s (-1).\n", errorMessage.c_str());
		return -1; // // internal error prior to connect
	}

	if (socket == INVALID_SOCKET) {
		// Trouble connecting -- fatal
		DUMP_OUT("ConnectToSocketSanityCheck: Failed to connect to %s:%d (1).\n");
		return 1;
	}

	closesocket(socket);

	return 0; // all good
}

/** Opens a socket to a given host and port.
 * Returns 0 if successful, 1 if there was a failure to connect to the target machine, -1 if there was an internal error.
 * Usage:
 *		Program [host name] [port]
 * If no hostname is given 'localhost' is used.
 * If no port is given, 80 is used.
 */
int main(int argc, const char **argv)
{

	std::string hostName = "localhost";
	unsigned short port = 80;

	if (argc > 1)
		hostName = argv[1];
	if (argc > 2)
		port = atoi(argv[2]);

	if (!initializeWindowsSockets()) {
		fprintf(stdout, "ConnectToSocketSanityCheck: Internal error: Unable to initialize sockets (-1).\n");
		return -1; // internal error
	}

	// Try connecting to the socket.
	// Try a number of times to survive transient failures.
	static const unsigned int MaxNumTries = 3;
	static const unsigned int PauseTimeBetweenTries = 2000; // ms
	unsigned int retryNum = 0;

	while (true) {

		int result = tryToConnect(hostName.c_str(), port);
		if (result == 0)
			break; // connected - all good

		retryNum++;

		if (retryNum == MaxNumTries) {
			uninitializeWindowsSockets();
			return result; // can't retry any more
		}

		Sleep(PauseTimeBetweenTries); // pause before trying again

	}

	uninitializeWindowsSockets();

	// All good
	DUMP_OUT("ConnectToSocketSanityCheck: Successfully connected to %s:%d (0).\n");
	return 0;
}


